@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("3.4.") or
-    raise "ruby lib version (3.4.3) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (3.4.5) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.4/x86_64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
   CONFIG["MINOR"] = "4"
-  CONFIG["TEENY"] = "3"
-  CONFIG["PATCHLEVEL"] = "32"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.4.3/bin/install -c'
+  CONFIG["TEENY"] = "5"
+  CONFIG["PATCHLEVEL"] = "51"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.4.5/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)34"
@@ -162,16 +162,16 @@ module RbConfig
   CONFIG["DOXYGEN"] = ""
   CONFIG["DOT"] = ""
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.4.3/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.4.3/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.4.3/bin/install -c  -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.4.5/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.4.5/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.4.5/bin/install -c  -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
   CONFIG["WINDRES"] = ""
   CONFIG["ARFLAGS"] = "rcD "
   CONFIG["try_header"] = ""
-  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 16.0.6\nTarget: amd64-unknown-openbsd7.7\nThread model: posix\nInstalledDir: /usr/bin"
+  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 19.1.7\nTarget: amd64-unknown-openbsd7.7\nThread model: posix\nInstalledDir: /usr/bin"
   CONFIG["CC_VERSION"] = "$(CC) --version"
   CONFIG["CSRCFLAG"] = ""
   CONFIG["COUTFLAG"] = "-o "
@@ -185,7 +185,7 @@ module RbConfig
   CONFIG["CPPFLAGS"] = "-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include $(DEFS) $(cppflags)"
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -fstack-protector-strong -Wl,-E -Wl,--no-as-needed -lpthread -lc++abi"
   CONFIG["CFLAGS"] = "-O2 -pipe -g $(cflags) -fPIC"
-  CONFIG["STRIP"] = "strip -A -n"
+  CONFIG["STRIP"] = "strip -S -x"
   CONFIG["RANLIB"] = "ranlib"
   CONFIG["OBJDUMP"] = "objdump"
   CONFIG["OBJCOPY"] = "objcopy"
